@@ -1,6 +1,8 @@
 import dbus
 import logging
 from dbus.mainloop.glib import DBusGMainLoop, threads_init
+import gi
+gi.require_version('UDisks', '2.0')
 from gi.repository import Gio, GLib, UDisks
 from usbcreator.backends.base import Backend
 from usbcreator import misc
