@@ -14,9 +14,12 @@
 
 import subprocess
 import os
-
 import gettext
 import logging
+
+import gi
+gi.require_version('Gdk', '3.0')
+gi.require_version('Gtk', '3.0')
 from gi.repository import GObject
 from gi.repository import GLib
 from gi.repository import Gio
@@ -30,10 +33,8 @@ from usbcreator import misc
 
 if 'USBCREATOR_LOCAL' in os.environ:
     ui_path = os.path.join(os.getcwd(), 'gui/usbcreator-gtk.ui')
-    eula_path = os.path.join(os.getcwd(), 'gui/ubuntu-nexus7-USAGE-NOTICE-en.txt')
 else:
     ui_path = '/usr/share/usb-creator/usbcreator-gtk.ui'
-    eula_path = '/usr/share/usb-creator/ubuntu-nexus7-USAGE-NOTICE-en.txt'
 
 Gdk.threads_init()
 
@@ -60,11 +61,9 @@ class GtkFrontend(Frontend):
         from dbus.mainloop.glib import DBusGMainLoop
         DBusGMainLoop(set_as_default=True)
 
-    def __init__(self, backend, img=None, persistent=True,
-                 allow_system_internal=False):
+    def __init__(self, backend, img=None, allow_system_internal=False):
 
         self.allow_system_internal = allow_system_internal
-        self.fastboot_mode = backend.__class__.__name__ == 'FastbootBackend'
 
         self.all_widgets = set()
 
@@ -101,11 +100,14 @@ class GtkFrontend(Frontend):
         self.failed_exit.connect('clicked', lambda x: self.failed_exit.hide())
         self.progress_cancel_button.connect('clicked', lambda x: self.warning_dialog.show())
 
+        # Set warning dialog transient window
+        self.warning_dialog.set_transient_for(self.install_window)
+
         def format_value(scale, value):
             return misc.format_mb_size(value)
-        self.persist_value.set_adjustment(
-            Gtk.Adjustment.new(0, 0, 100, 1, 10, 0))
-        self.persist_value.connect('format-value', format_value)
+
+        # TODO: remove format button
+        self.format_dest.hide()
 
         # Connect to backend signals.
         self.backend = backend
@@ -121,39 +123,18 @@ class GtkFrontend(Frontend):
         self.backend.install_progress_pulse_stop_cb = self.progress_pulse_stop
         self.backend.retry_cb = self.retry
         self.backend.target_changed_cb = self.update_target
-        self.backend.format_ended_cb = self.format_ended
-        self.backend.format_failed_cb = self.format_failed
 
         # Pulse state.
         self.pulsing = False
 
-        # Hide erase in fastboot mode
-        if self.fastboot_mode:
-            self.format_dest.hide()
-            self.intro_label.set_text(_('To run Ubuntu on a portable device, it needs to be set up first.'))
-            self.label2.set_text(_('Source disc image (.img):'))
-            self.label3.set_text(_('Target device:'))
-            self.window.set_title(_('Ubuntu Core Installer'))
-            self.button_install.set_label(_('Install Ubuntu Core'))
-            self.button_help.set_label(_('Legal'))
-            self.button_help.connect('clicked', self.show_eula)
-            self.eula_dialog.add_buttons(
-                Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
-                Gtk.STOCK_OK, Gtk.ResponseType.OK
-            )
-            with open(eula_path, 'r') as f:
-                text = ''.join(f.readlines())
-            self.eula_text.get_buffer().set_text(text)
-        else:
-            # we currently do not have help
-            self.button_help.hide()
-            #self.button_help.connect('clicked', lambda x: Gtk.show_uri(self.button_help.get_screen(),
-            #                                                           'ghelp:usb-creator',
-            #                                                           Gtk.get_current_event_time()))
+        # we currently do not have help
+        self.button_help.hide()
+        #self.button_help.connect('clicked', lambda x: Gtk.show_uri(self.button_help.get_screen(),
+        #                                                           'ghelp:usb-creator',
+        #                                                           Gtk.get_current_event_time()))
             
         self.setup_sources_treeview()
         self.setup_targets_treeview()
-        self.persist_vbox.set_sensitive(False)
 
         # Pre-populate the source view.
         if img is not None:
@@ -171,10 +152,6 @@ class GtkFrontend(Frontend):
         # Sets first pre-populated image as current in the backend
         self.selection_changed_source(self.source_treeview.get_selection())
 
-        if not persistent:
-            self.persist_disabled.set_active(True)
-            self.persist_vbox.hide()
-
         self.window.show()
         selection = self.source_treeview.get_selection()
         selection.connect('changed', self.selection_changed_source)
@@ -301,14 +278,6 @@ class GtkFrontend(Frontend):
             logging.debug('No target selected.')
             return ''
 
-    def get_persistence(self):
-        if self.persist_enabled.get_active() and \
-            self.persist_enabled.get_state() != Gtk.StateType.INSENSITIVE:
-            val = self.persist_value.get_value()
-            return int(val)
-        else:
-            return 0
-
     def get_gnome_drive(self, dev):
         try:
             monitor = Gio.VolumeMonitor.get()
@@ -367,8 +336,6 @@ class GtkFrontend(Frontend):
         cell_name.set_property('ellipsize', Pango.EllipsizeMode.END)
         cell_pixbuf = Gtk.CellRendererPixbuf()
         column_name = Gtk.TreeViewColumn(_('CD-Drive/Image'))
-        if self.fastboot_mode:
-            column_name = Gtk.TreeViewColumn(_('Image'))
         column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         column_name.set_resizable(True)
         column_name.set_expand(True)
@@ -379,16 +346,15 @@ class GtkFrontend(Frontend):
         column_name.set_cell_data_func(cell_name, column_data_func, 0)
         column_name.set_cell_data_func(cell_pixbuf, pixbuf_data_func, None)
 
-        if not self.fastboot_mode:
-            cell_version = Gtk.CellRendererText()
-            cell_version.set_property('ellipsize', Pango.EllipsizeMode.END)
-            column_name = Gtk.TreeViewColumn(_('OS Version'), cell_version)
-            column_name.set_cell_data_func(cell_version, column_data_func, 1)
-            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
-            column_name.set_resizable(True)
-            column_name.set_expand(True)
-            column_name.set_min_width(75)
-            self.source_treeview.append_column(column_name)
+        cell_version = Gtk.CellRendererText()
+        cell_version.set_property('ellipsize', Pango.EllipsizeMode.END)
+        column_name = Gtk.TreeViewColumn(_('OS Version'), cell_version)
+        column_name.set_cell_data_func(cell_version, column_data_func, 1)
+        column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        column_name.set_resizable(True)
+        column_name.set_expand(True)
+        column_name.set_min_width(75)
+        self.source_treeview.append_column(column_name)
 
         cell_size = Gtk.CellRendererText()
         cell_size.set_property('ellipsize', Pango.EllipsizeMode.END)
@@ -444,17 +410,6 @@ class GtkFrontend(Frontend):
 
         target = self.backend.targets[udi]
 
-        # Update persistence maximum value.
-        if not self.fastboot_mode:
-            self.persist_vbox.set_sensitive(False)
-            self.persist_enabled_vbox.set_sensitive(False)
-            persist_mb = target['persist'] / 1024 / 1024
-            if persist_mb > misc.MIN_PERSISTENCE:
-                    self.persist_vbox.set_sensitive(True)
-                    self.persist_enabled_vbox.set_sensitive(True)
-                    self.persist_value.set_range(misc.MIN_PERSISTENCE, persist_mb)
-                    self.persist_value.set_value(misc.MIN_PERSISTENCE)
-
         # Update install button state.
         status = target['status']
         source = self.backend.get_current_source()
@@ -469,17 +424,10 @@ class GtkFrontend(Frontend):
         self.open_dest.hide()
         if status == misc.CANNOT_USE:
             msg = _('The device is not large enough to hold this image.')
-        elif status == misc.NEED_SPACE:
-            msg = _('There is not enough free space for this image.')
-            self.open_dest.show()
         else:
             msg = ''
         self.dest_status.set_text(msg)
 
-    def format_ended(self):
-        selection = self.dest_treeview.get_selection()
-        self.selection_changed_target(selection)
-
     def selection_changed_source(self, selection):
         model, iterator = selection.get_selected()
         if not iterator:
@@ -489,10 +437,6 @@ class GtkFrontend(Frontend):
         self.selection_changed_target(self.dest_treeview.get_selection())
 
     def selection_changed_target(self, selection):
-        '''The selected partition has changed and the bounds on the persistence
-        slider need to be changed, or the slider needs to be disabled, to
-        reflect the amount of free space on the partition.'''
-
         model, iterator = selection.get_selected()
         if not iterator:
             return
@@ -500,23 +444,10 @@ class GtkFrontend(Frontend):
         if udi:
             self.update_target(udi)
 
-        if self.fastboot_mode:
-            return
-
         dev = self.backend.targets[udi]
         p = dev['parent']
         if p and p in self.backend.targets:
             dev = self.backend.targets[p]
-        if dev['formatting']:
-            self.format_dest.set_sensitive(False)
-            self.format_dest_label.set_text(_('Erasing...'))
-            self.format_dest_spinner.show()
-            self.format_dest_spinner.start()
-        else:
-            self.format_dest.set_sensitive(True)
-            self.format_dest_label.set_text(_('Erase Disk'))
-            self.format_dest_spinner.hide()
-            self.format_dest_spinner.stop()
 
     def setup_targets_treeview(self):
         def column_data_func(layout, cell, model, iterator, column):
@@ -537,12 +468,6 @@ class GtkFrontend(Frontend):
                     cell.set_property('text', dev['label'])
             elif column == 2:
                 cell.set_property('text', misc.format_size(dev['capacity']))
-            elif column == 3:
-                free = dev['free']
-                if free >= 0:
-                    cell.set_property('text', misc.format_size(free))
-                else:
-                    cell.set_property('text', '')
 
         def pixbuf_data_func(column, cell, model, iterator, data):
             if not self.backend:
@@ -582,49 +507,41 @@ class GtkFrontend(Frontend):
         column_name.set_cell_data_func(cell_name, column_data_func, 0)
         column_name.set_cell_data_func(cell_pixbuf, pixbuf_data_func, None)
 
-        if not self.fastboot_mode:
-            cell_name = Gtk.CellRendererText()
-            cell_name.set_property('ellipsize', Pango.EllipsizeMode.END)
-            column_name = Gtk.TreeViewColumn(_('Label'), cell_name)
-            column_name.set_cell_data_func(cell_name, column_data_func, 1)
-            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
-            column_name.set_resizable(True)
-            column_name.set_expand(True)
-            column_name.set_min_width(75)
-            self.dest_treeview.append_column(column_name)
+        cell_name = Gtk.CellRendererText()
+        cell_name.set_property('ellipsize', Pango.EllipsizeMode.END)
+        column_name = Gtk.TreeViewColumn(_('Label'), cell_name)
+        column_name.set_cell_data_func(cell_name, column_data_func, 1)
+        column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        column_name.set_resizable(True)
+        column_name.set_expand(True)
+        column_name.set_min_width(75)
+        self.dest_treeview.append_column(column_name)
 
-            cell_capacity = Gtk.CellRendererText()
-            cell_capacity.set_property('ellipsize', Pango.EllipsizeMode.END)
-            column_name = Gtk.TreeViewColumn(_('Capacity'), cell_capacity)
-            column_name.set_cell_data_func(cell_capacity, column_data_func, 2)
-            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
-            column_name.set_resizable(True)
-            column_name.set_expand(False)
-            column_name.set_min_width(75)
-            self.dest_treeview.append_column(column_name)
-
-            cell_free = Gtk.CellRendererText()
-            cell_free.set_property('ellipsize', Pango.EllipsizeMode.END)
-            column_name = Gtk.TreeViewColumn(_('Free Space'), cell_free)
-            column_name.set_cell_data_func(cell_free, column_data_func, 3)
-            column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
-            column_name.set_resizable(True)
-            column_name.set_expand(False)
-            column_name.set_min_width(75)
-            self.dest_treeview.append_column(column_name)
+        cell_capacity = Gtk.CellRendererText()
+        cell_capacity.set_property('ellipsize', Pango.EllipsizeMode.END)
+        column_name = Gtk.TreeViewColumn(_('Capacity'), cell_capacity)
+        column_name.set_cell_data_func(cell_capacity, column_data_func, 2)
+        column_name.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        column_name.set_resizable(True)
+        column_name.set_expand(False)
+        column_name.set_min_width(75)
+        self.dest_treeview.append_column(column_name)
 
     def add_file_source_dialog(self, *args):
         filename = ''
-        chooser = Gtk.FileChooserDialog(title=None,action=Gtk.FileChooserAction.OPEN,
-            buttons=(Gtk.STOCK_CANCEL,Gtk.ResponseType.CANCEL,Gtk.STOCK_OPEN,Gtk.ResponseType.OK))
+        chooser = Gtk.FileChooserDialog(title=None, parent=self.window,
+                                        action=Gtk.FileChooserAction.OPEN,
+                                        buttons=(Gtk.STOCK_CANCEL,
+                                                 Gtk.ResponseType.CANCEL,
+                                                 Gtk.STOCK_OPEN,
+                                                 Gtk.ResponseType.OK))
         def _add_filter(p, n):
             filter = Gtk.FileFilter()
             filter.add_pattern(p)
             filter.set_name(n)
             chooser.add_filter(filter)
 
-        if not self.fastboot_mode:
-            _add_filter('*.iso', _('CD Images'))
+        _add_filter('*.iso', _('CD Images'))
         _add_filter('*.img', _('Disk Images'))            
 
         folder = os.path.expanduser('~')
@@ -644,34 +561,38 @@ class GtkFrontend(Frontend):
                     return
                 iterator = model.iter_next(iterator)
             self.backend.add_image(filename)
-        elif response == Gtk.ResponseType.CANCEL:
+        else:
             chooser.destroy()
 
     def install(self, widget):
         source = self.get_source()
         target = self.get_target()
-        persist = self.get_persistence()
-        if self.fastboot_mode and not misc.check_eula():    
-            if self.show_eula() == Gtk.ResponseType.CANCEL:
-                return
-        # TODO evand 2009-07-31: Make these the default values in the
-        # GtkBuilder file.
-        starting_up = _('Starting up...')
-        self.progress_title.set_markup('<big><b>' + starting_up + '</b></big>')
-        self.progress_info.set_text('')
         if source and target:
+            d = Gtk.MessageDialog(transient_for=self.window, modal=True,
+                    message_type=Gtk.MessageType.QUESTION, buttons=Gtk.ButtonsType.YES_NO)
+            d.set_markup(_('Are you sure you want to write the disc image to the device?'))
+            d.format_secondary_markup(_('All existing data will be lost.'))
+            response = d.run()
+            d.destroy()
+            if response != Gtk.ResponseType.YES:
+                return
+
+            starting_up = _('Starting up...')
+            self.progress_title.set_markup('<big><b>' + starting_up + '</b></big>')
+            self.progress_info.set_text('')
+
             self.install_window.show()
             self.window.hide()
             self.unity.show_progress()
             self.delete_timeout(self.update_loop)
             try:
-                self.backend.install(source, target, persist,
+                self.backend.install(source, target,
                                      allow_system_internal=self.allow_system_internal)
             except:
                 self._fail()
 
     @thread_wrap
-    def progress(self, complete, remaining, speed):
+    def progress(self, complete):
         if self.pulsing:
             self.progress_info.set_text('')
             return
@@ -680,16 +601,10 @@ class GtkFrontend(Frontend):
             complete = 100
         self.progress_bar.set_fraction(complete / 100.0)
         self.unity.set_progress(complete / 100.0)
-        if remaining and speed:
-            # TODO evand 2009-07-24: Could use a time formatting function
-            # like our human size function.
-            mins = int(remaining / 60)
-            secs = int(remaining % 60)
-            text = _('%d%% complete (%dm%ss remaining)') % \
-                    (complete, mins, secs)
-            self.progress_info.set_text(text)
-        else:
+        if complete < 100:
             self.progress_info.set_text(_('%d%% complete') % complete)
+        else:
+            self.progress_info.set_text(_('Finishing...'))
 
     @thread_wrap
     def progress_message(self, message):
@@ -717,13 +632,6 @@ class GtkFrontend(Frontend):
         retry_dialog.destroy()
         return response == Gtk.ResponseType.YES
 
-    def show_eula(self, unused=None):
-        response = self.eula_dialog.run()
-        if response == Gtk.ResponseType.OK:
-            misc.check_eula(True)
-        self.eula_dialog.hide()
-        return response
-
     def quit(self, *args):
         self.backend.cancel_install()
         if Gtk.main_level() > 0:
@@ -741,13 +649,13 @@ class GtkFrontend(Frontend):
         self.unity.show_progress(False)
         if not message:
             message = _('Installation failed.')
+        self.failed_dialog.set_transient_for(self.install_window)
         self.failed_dialog_label.set_text(message)
         self.failed_dialog.run()
         Gtk.main_quit()
 
     @thread_wrap
     def success(self):
-        self.backend.unmount()
         self.warning_dialog.hide()
         self.install_window.hide()
         self.unity.show_progress(False)
@@ -766,10 +674,7 @@ class GtkFrontend(Frontend):
         except dbus.DBusException:
             logging.exception('Error checking for kvm:')
 
-        if self.fastboot_mode:
-            self.finished_dialog_label.set_text(
-                _('Installation is complete. Your device is rebooting into Ubuntu Core.'))
-            self.kvm_test.hide()
+        self.finished_dialog.set_transient_for(self.install_window)
         self.finished_dialog.run()
         self.backend.shutdown()
         Gtk.main_quit()
@@ -781,34 +686,8 @@ class GtkFrontend(Frontend):
         dialog.run()
         dialog.destroy()
 
-    @thread_wrap
-    def format_failed(self, message):
-        # TODO sort through error types (message.get_dbus_name()) in backend,
-        # individual functions in frontend for each error type.
-        d = Gtk.MessageDialog(transient_for=self.window, modal=True,
-                              message_type=Gtk.MessageType.ERROR,
-                              buttons=Gtk.ButtonsType.CLOSE)
-        d.set_markup(str(message))
-        d.run()
-        d.destroy()
-
     def format_dest_clicked(self, *args):
-        model, iterator = self.dest_treeview.get_selection().get_selected()
-        if not iterator:
-            return
-        udi = model[iterator][0]
-        d = Gtk.MessageDialog(transient_for=self.window, modal=True,
-                message_type=Gtk.MessageType.QUESTION, buttons=Gtk.ButtonsType.YES_NO)
-        # TODO information about the device we're about to format.
-        d.set_markup(_('Are you sure you want to erase the entire disk?'))
-        response = d.run()
-        d.destroy()
-        if response == Gtk.ResponseType.YES:
-            self.format_dest.set_sensitive(False)
-            self.format_dest_label.set_text(_('Erasing...'))
-            self.format_dest_spinner.show()
-            self.format_dest_spinner.start()
-            self.backend.format(udi)
+        return
 
     def open_dest_folder(self, *args):
         model, iterator = self.dest_treeview.get_selection().get_selected()
